import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.*;
import java.util.*;

/**
 * コントロールクラス.
 */
public class KCtrlPZL extends KCtrl implements KConstant {

	/**
	 * フェーズ.
	 */
	private final int PHASE_INIT		= 0;
	private final int PHASE_TITLE		= 1;
	private final int PHASE_STAGE_START	= 2;
	private final int PHASE_RUN			= 3;
	private final int PHASE_FAIL		= 4;
	private final int PHASE_END			= 5;
	private final int PHASE_STOP		= -1;

	/**
	 * パネル.
	 */
	private		KPanel				panel				= null;

	/**
	 * イメージ.
	 */
	private		BufferedImage		imageBackground		= null;
	private		BufferedImage		imageBlock00_01		= null;
	private		BufferedImage		imageBlock00_02		= null;
	private		BufferedImage		imageBlock00_03		= null;
	private		BufferedImage		imageBlock01		= null;
	private		BufferedImage		imageBlock02		= null;
	private		BufferedImage		imageBlock03		= null;
	private		BufferedImage		imageBlock04		= null;
	private		BufferedImage		imageBlockAA		= null;

	/**
	 * スプライト.
	 * 全体的に表示等されるスプライトは、ここで管理する。
	 */
	private		KSpriteBackground	spriteBackground	= null;

	/**
	 * テキスト.
	 */
	private		KTextTitle			kTextTitle			= null;
	private		KText				kTextStart			= null;
	private		KText				kTextBy				= null;
	private		KText				kTextEnd			= null;
	private		KText				kTextScoreHigh		= null;
	private		KText				kTextScore			= null;
	private		KText				kTextStage			= null;
	private		KText				kTextMessage		= null;

	/**
	 * スプライトブロック一覧.
	 */
	private		KSpriteBlock01[][]	sprBlocks			= new KSpriteBlock01[BLOCK_ROW_COUNT][BLOCK_COL_COUNT];

	/**
	 * フェーズ.
	 */
	private		int					phase				= PHASE_INIT;

	/**
	 * スコア.
	 */
	private		int					score				= 0;

	/**
	 * ハイスコア.
	 */
	private		int					scoreHigh			= 10000;

	/**
	 * ステージ.
	 */
	private		int					stage				= 1;

	/**
	 * 時間.
	 */
	private		int					time				= 0;

	/**
	 * マウスアダプタ.
	 */
	private		KFPMouseAdapter		kfpMouseAdapter		= null;

	/**
	 * コンストラクタ.
	 * @param con コンテナ
	 */
	public KCtrlPZL(Container con){

		super(con);

		try{

			// パネルを生成
			panel = new KPanel(this);

			// パネルのサイズを設定
			panel.setSize(KConstant.WIDTH, KConstant.HEIGHT);
			panel.setPreferredSize(new Dimension(KConstant.WIDTH, KConstant.HEIGHT));

			// スプライト層の数を設定
			panel.setLayoutCount(LAYOUT_COUNT);

			// イメージを取得
			imageBackground = KImage.getImage("imageBackground.gif");
			imageBlock00_01 = KImage.getImage("imageBlock00_01.gif");
			imageBlock00_02 = KImage.getImage("imageBlock00_02.gif");
			imageBlock00_03 = KImage.getImage("imageBlock00_03.gif");
			imageBlock01 = KImage.getImage("imageBlock01.gif");
			imageBlock02 = KImage.getImage("imageBlock02.gif");
			imageBlock03 = KImage.getImage("imageBlock03.gif");
			imageBlock04 = KImage.getImage("imageBlock04.gif");
			imageBlockAA = KImage.getImage("imageBlockAA.gif");

			// イメージをスプライトに登録する
			KSpriteBackground.setImage(KSpriteBackground.class, imageBackground);
			KSpriteBlock00.setImage(KSpriteBlock00.class, new BufferedImage[]{imageBlock00_01, imageBlock00_02, imageBlock00_03});
			KSpriteBlock01.setImage(KSpriteBlock01.class, new BufferedImage[]{imageBlock01, imageBlock02, imageBlock03, imageBlock04});
			KSpriteBlockAA.setImage(KSpriteBlockAA.class, imageBlockAA);

			// 背景生成
			spriteBackground = new KSpriteBackground(panel);

			//
			// 各テキストを生成
			//

			kTextTitle = new KTextTitle(panel, 0, 0);

			kTextStart = new KText(panel, 176, 400);
			kTextStart.setFont(new Font("HG丸ｺﾞｼｯｸM-PRO", Font.PLAIN, 16));
			kTextStart.setText("CLICK MOUSE BUTTON");

			kTextBy = new KText(panel, 200, 480);
			kTextBy.setFont(new Font("ＭＳ ゴシック", Font.PLAIN, 12));
			kTextBy.setText("PROGRAMED  BY  KIN");

			kTextEnd = new KText(panel, 250, 300);
			kTextEnd.setFont(new Font("HG丸ｺﾞｼｯｸM-PRO", Font.PLAIN, 16));
			kTextEnd.setText("GAME OVER");

			kTextScoreHigh = new KText(panel, 50, 20);
			panel.addText("HIGH-SCORE", kTextScoreHigh);

			kTextScore = new KText(panel, 200, 20);
			panel.addText("SCORE", kTextScore);

			kTextStage = new KText(panel, 350, 20);
			panel.addText("STAGE", kTextStage);

			kTextMessage = new KText(panel, 240, 100);
			kTextMessage.setFont(new Font("HG丸ｺﾞｼｯｸM-PRO", Font.BOLD, 24));

			// マウスアダプタ生成
			kfpMouseAdapter = new KFPMouseAdapter();

			// マウスアダプタをパネルに登録
			panel.addMouseListener(kfpMouseAdapter);
			panel.addMouseMotionListener(kfpMouseAdapter);

		// エラーの場合、
		}catch(Exception ex){
			// スタックトレース出力
			ex.printStackTrace();
			// エラーを表示
			JOptionPane.showMessageDialog(null, ex.toString());
		}

	} // end KCtrlPZL

	/**
	 * 開始.
	 * <pre>
	 * 当オブジェクトの処理を開始します。
	 * </pre>
	 */
	public void start(){

		// パネルにフォーカスを当てる
		panel.requestFocus();

		// パネルを開始する
		panel.start();

		// 初期化フェーズへ
		this.phase = PHASE_INIT;

	} // end start

	/**
	 * 実行.
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * </pre>
	 */
	public void run(){

		// フェーズ分け
		switch(this.phase){

			// 初期化フェーズ
			case PHASE_INIT:

				// スコア
				score = 0;

				// ステージ
				stage = 1;

				// 時間初期化
				time = 0;

				// ブロックのスプライト層を初期化
				panel.initKSpriteLstInLayout(LAYOUT_BLK0);
				panel.initKSpriteLstInLayout(LAYOUT_BLK1);

				// 背景のスプライト層を初期化
				panel.initKSpriteLstInLayout(LAYOUT_HAIKEI0);
				panel.initKSpriteLstInLayout(LAYOUT_HAIKEI1);
				panel.initKSpriteLstInLayout(LAYOUT_HAIKEI2);

				// スプライトグループ初期化
				panel.initSpriteGroup();

				// 背景を登録
				panel.addSprite(LAYOUT_HAIKEI0, spriteBackground);

				// タイトル表示
				panel.addText("TITLE", kTextTitle);
				panel.addText("START", kTextStart);
				panel.addText("BY", kTextBy);

				// 各テキストを最新化
				kTextScoreHigh.setText("HIGH-SCORE : " + Integer.toString(scoreHigh));

				// スコア初期化
				kTextScore.setText("SCORE : " + Integer.toString(score));

				// ステージ初期化
				kTextStage.setText("STAGE : " + Integer.toString(stage));

				//
				// 壁を作成
				//

				// 縦ループ
				for(int row = 0; row < 17; row++){

					KSpriteBlockAA kSpriteBlockAA = new KSpriteBlockAA(panel);
					panel.addSprite(LAYOUT_HAIKEI1, kSpriteBlockAA);
					kSpriteBlockAA.setY(row * 32);

					kSpriteBlockAA = new KSpriteBlockAA(panel);
					panel.addSprite(LAYOUT_HAIKEI1, kSpriteBlockAA);
					kSpriteBlockAA.setX(512);
					kSpriteBlockAA.setY(row * 32);

				} // ewd for 縦ループ

				// 横ループ
				for(int col = 0; col < 17; col++){

					KSpriteBlockAA kSpriteBlockAA = new KSpriteBlockAA(panel);
					panel.addSprite(LAYOUT_HAIKEI1, kSpriteBlockAA);
					kSpriteBlockAA.setX(col * 32);
					kSpriteBlockAA.setY(512);

				} // ewd for 横ループ

				// タイトル表示フェーズへ
				this.phase = PHASE_TITLE;

				break;

			// タイトル表示フェーズ
			case PHASE_TITLE:

				break;

			// ステージ開始フェーズ
			case PHASE_STAGE_START:

				time++;

				// READY
				if(time == 1){

					// タイトルを削除
					panel.removeText("TITLE");
					panel.removeText("START");
					panel.removeText("BY");

					// ブロック初期化
					initBlocks();

					// 場所修正
					kTextMessage.setX(216);

					// READY
					kTextMessage.setText("READY");

					// READY表示
					panel.addText("MESSAGE", kTextMessage);

				// GO!
				}else if(time == 120){

					// 場所修正
					kTextMessage.setX(240);

					// GO!
					kTextMessage.setText("GO!");

				// 本当にGO!
				}else if(time == 180){

					// GO!削除
					panel.removeText("MESSAGE");

					// 通常フェーズへ
					this.phase = PHASE_RUN;

				}

				break;

			// 通常フェーズ
			case PHASE_RUN:

				break;

			// 失敗フェーズ
			case PHASE_FAIL:

				break;

			// ゲームオーバーフェーズ
			case PHASE_END:

				break;

			default:
				break;

		} // end switch フェーズ分け

	} // end run

	/**
	 * パネル取得.
	 * @return KPanel パネル
	 */
	public KPanel getKPanel(){

		// パネルをリターン
		return this.panel;

	}

	/**
	 * マウスイベント.
	 * <pre>
	 * マウスイベントによる制御を行います。
	 * </pre>
	 * @param me マウスイベント
	 */
	public void mouseEvent(MouseEvent me){

		// フェーズ分け
		switch(this.phase){

			// タイトル画面フェーズ
			case PHASE_TITLE:

				// クリック以外、
				if(me.getID() != MouseEvent.MOUSE_CLICKED){
					// リターン
					return;
				}

				// ステージ開始フェーズへ
				phase = PHASE_STAGE_START;

				break;

			default:
				break;

		} // end switch フェーズ分け

	} // end mouseEvent

	/**
	 * ブロック初期化.
	 */
	private void initBlocks(){

		// 行ループ
		for(int row = 0; row < BLOCK_ROW_COUNT; row++){

			// 列ループ
			for(int col = 0; col < BLOCK_COL_COUNT; col++){

				// スプライトブロックを生成
				KSpriteBlock01 sprBlock =  new KSpriteBlock01(panel, getNewBlock());

				// スプライトブロック一覧に設定
				sprBlocks[row][col] = sprBlock;

				// ブロック位置設定
				sprBlock.setY(POS_Y + row * 32);
				sprBlock.setX(POS_X + col * 32);

				// パネルにブロックを追加
				panel.addSprite(LAYOUT_BLK0, sprBlock);

			} // end for 列ループ

		} // end for 行ループ

	} // end initBlocks

	/**
	 * 新しいブロック色を取得.
	 * <pre>
	 * ステージを考慮したブロック色を返します。
	 * </pre>
	 * @return int 新しいブロック色
	 */
	private int getNewBlock(){
		return (int)(Math.random() * (stage + 2));
	}

	/**
	 * マウスイベント.
	 */
	private class KFPMouseAdapter extends MouseAdapter {

		/**
		 * マウスクリック.
		 */
		public void mouseClicked(MouseEvent me){

			// スーパークラス呼び出し
			super.mouseClicked(me);

			// マウスイベント呼び出し
			KCtrlPZL.this.mouseEvent(me);

		} // end mouseClicked

	} // end KFPMouseAdapter

}
